# frozen_string_literal: true

class Edaemon::Server
  PID_FILE_PATH = './edaemon.pid'
  LOG_FILE_PATH = './edaemon.log'

  STARTED = :started
  STOPPED = :stopped

  Error = Class.new(StandardError)
  ApplicationAllreadyRunnedError = Class.new(Error)
  BlockNotSettedError = Class.new(Error)

  def initialize(block: nil)
    @block = block
  end

  # TODO: Конфиг лога
  # TODO: Конфиг пид файла
  def start
    # TODO: simplecov raise почему не подсвечивается
    raise ApplicationAllreadyRunnedError if status == STARTED
    raise BlockNotSettedError unless block

    pid = fork_block!
    save_pid!(pid: pid)
    pid
  end

  def status
    pid_runned?(pid: saved_pid) ? STARTED : STOPPED
  end

  attr_accessor :block

  private

  def saved_pid
    return unless File.exist?(PID_FILE_PATH)

    File.open(PID_FILE_PATH).read.to_i
  end

  def pid_runned?(pid:)
    Edaemon::Helper::Process.pids.include?(pid)
  end

  def fork_block!
    # TODO: Переделать не на call из блока а на простой вызов переданного метода
    Kernel.fork do
      STDOUT.reopen(File.open(LOG_FILE_PATH, 'a'))
      STDOUT.sync = true
      block.call
    end
  end

  def save_pid!(pid:)
    File.open(PID_FILE_PATH, 'w') { |f| f.write("#{pid}\n") }
  end
end
